from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from ...continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from pint import UnitRegistry
ureg = UnitRegistry()

from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST

    
class SDoFShaft(ComposedSystem):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertiafrom sympy import *
from sympy.plotting import plot
from pylatex import Document, Section, Subsection, Subsubsection, Command, Itemize, Package, HorizontalSpace, Description, Marker
from pylatex.section import Paragraph, Chapter
from pylatex.utils import italic, NoEscape

from dynpy.utilities.report import * #to do tworzenia raportu pozniej

from dynpy.utilities.adaptable import *
import pypandoc as ppd
import importlib


from pylatex import Package, Document, NoEscape, Command, Section
from sympy.physics.mechanics import *
from dynpy.solvers.linear import * #ODESystem here
from dynpy.models.mechanics.pendulum import *

from dynpy.models.mechanics.pendulum import *  #skad zaciagasz modele
 
mechanics_printing() #mechaniczny zapis pochodnej to kropki, ta linijka zmienia z d/dt na kropkis
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'shaft_sdof_scheme.png'
    real_name = 'ddof_shaft_real.png'
    detail_scheme_name = 'parallel_key_load.png'
    detail_real_name = 'shaft_with_key.png'

    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    I = Symbol('I', positive=True)
    l_1 = Symbol('l_1', positive=True)
    l_2 = Symbol('l_2', positive=True)
    I_1 = Symbol('I_1', positive=True)
    I_2 = Symbol('I_2', positive=True)
    Ms = Symbol('M_s', positive=True)
    Omega = Symbol('Omega', positive=True)

    theta = dynamicsymbols('theta')
    phi = dynamicsymbols('\\varphi')
    def __init__(self,
                 l0=None,
                 G=None,
                 I=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 Ms=None,
                 phi=None,
                 theta=None,
                 ivar=Symbol('t'),
                 qs=None,
                 **kwargs):
        if G is not None: self.G = G

        if I is not None: self.I = I
        if Ms is not None: self.Ms = Ms
        #if Omega is not None: self.Omega = Omega
        if l_1 is not None: self.l_1 = l_1
        if l_2 is not None: self.l_2 = l_2
        if I_1 is not None: self.I_1 = I_1
        if I_2 is not None: self.I_2 = I_2
        if phi is not None: self.phi = phi
        if theta is not None: self.theta = theta
        self.qs = [self.phi]
        self._init_from_components(**kwargs)

    @property
    def components(self):
        components = {}

        self.k_1 = (self.G * self.I_1) / self.l_1
        self.k_2 = (self.G * self.I_2) / self.l_2

        self.disc_1 = Disk(self.I, pos1=self.phi, qs=self.qs)
        self.spring_2 = Spring(self.k_1 * self.k_2 / (self.k_2 + self.k_1),
                               pos1=self.phi,
                               pos2=self.theta,
                               qs=self.qs)  # right spring
        self.moment = Force(self.Ms, pos1=self.phi, qs=self.qs)


        components['disc_1'] = self.disc_1
        components['spring_2'] = self.spring_2
        components['moment'] = self.moment

        return components


    def symbols_description(self):
        self.sym_desc_dict = {
            self.I: r'Moment of Inertia',
            self.k_1: r'',
            self.k_2: r'',
        }
        return self.sym_desc_dict

    def get_default_data(self):

        m0, l0, G, l = symbols('m_0 l_0 G l', positive=True)
        theta0, Omega = symbols('theta_0, Omega', positive=True)

        default_data_dict = {
            self.I: [S.Half * m0 * (l0**2) * no for no in range(1, 3)],
            self.I_1: [S.Half**(no) * (l0**4) for no in range(1, 8)],
            self.I_2: [S.Half**no * (l0**4) for no in range(1, 8)],
            self.l_1: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.l_2: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.theta: [theta0 * cos(Omega * self.ivar)],
        }

        return default_data_dict
    
    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST,
        mech_comp.MaxStaticForce,
        mech_comp.MaxDynamicForce,
        mech_comp.StaticKeyLength,
        mech_comp.DynamicKeyLength,
        mech_comp.DynamicTorqueComponent

        ]

        return comp_list

    def disc_force(self):
        t = self.ivar
        return self.I * self.steady_solution()[0].diff(t, t)

    def max_static_force(self):
        d = Symbol('d', positive=True)
        return 2 * self.Ms / d

    def max_dynamic_force(self):
        d, theta0, Omega = symbols('d theta_0, Omega', positive=True)
#         return self.frequency_response_function(
#             self.natural_frequencies()[0]) * self.stiffness_matrix()[0]
        return self.subs({self.theta:theta0*cos(Omega*self.ivar)}).frequency_response_function() * self.spring_2.stiffness *2/d + self.max_static_force()

    def max_static_bearing_force(self):
        d = Symbol('d', positive=True)
        return abs(2 * self.static_load()[0] / d)

    def max_dynamic_bearing_force(self):
        d = Symbol('d', positive=True)
        acc_amp = self.frequency_response_function() * self.Omega**2

        return abs(
            2 * (self.I * acc_amp) /
            d) + self.max_static_bearing_force()  #.subs(self._given_data)

    def static_key_length(self):
        kd = Symbol('k_d', positive=True)
        h = Symbol('h', positive=True)
        return (2 * self.max_static_force()) / (kd * h)

    def dynamic_key_length(self):
        kd = Symbol('k_d', positive=True)
        h = Symbol('h', positive=True)
        return (2 * self.max_dynamic_force()) / (kd * h)
    
    def dynamic_torque(self):
        t=self.ivar
        steady=self._fodes_system.steady_solution[0]
        torque=steady.diff(t,t)*self.I+self.Ms
        return torque
        
class SDoFShaftHarmonicExcitation(SDoFShaft):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertia
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'shaft_sdof_scheme.png'
    real_name = 'ddof_shaft_real.png'
    detail_scheme_name = 'parallel_key_load.png'
    detail_real_name = 'shaft_with_key.png'

    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    I = Symbol('I', positive=True)
    l_1 = Symbol('l_1', positive=True)
    l_2 = Symbol('l_2', positive=True)
    I_1 = Symbol('I_1', positive=True)
    I_2 = Symbol('I_2', positive=True)
    Ms = Symbol('M_s', positive=True)
    Omega = Symbol('Omega', positive=True)

    theta = dynamicsymbols('theta')
    theta0=Symbol('theta_0', positive=True)
    phi = dynamicsymbols('\\varphi')
    def __init__(self,
                 l0=None,
                 G=None,
                 I=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 Ms=None,
                 phi=None,
                 theta0=None,
                 Omega=None,
                 ivar=Symbol('t'),
                 qs=None,
                 **kwargs):
        if G is not None: self.G = G

        if I is not None: self.I = I
        if Ms is not None: self.Ms = Ms
        if Omega is not None: self.Omega = Omega
        if l_1 is not None: self.l_1 = l_1
        if l_2 is not None: self.l_2 = l_2
        if I_1 is not None: self.I_1 = I_1
        if I_2 is not None: self.I_2 = I_2
        if phi is not None: self.phi = phi
        if theta0 is not None: self.theta0 = theta0
        self.qs = [self.phi]
        self._init_from_components(**kwargs)

    @property
    def components(self):
        components = {}

        self.k_1 = (self.G * self.I_1) / self.l_1
        self.k_2 = (self.G * self.I_2) / self.l_2

        self.disc_1 = Disk(self.I, pos1=self.phi, qs=self.qs)
        self.spring_2 = Spring(self.k_1 * self.k_2 / (self.k_2 + self.k_1),
                               pos1=self.phi,
                               pos2=self.theta0*cos(self.Omega*self.ivar),
                               qs=self.qs)  # right spring
        self.moment = Force(self.Ms, pos1=self.phi, qs=self.qs)


        components['disc_1'] = self.disc_1
        components['spring_2'] = self.spring_2
        components['moment'] = self.moment

        return components
    
    def get_default_data(self):

        m0, l0, G, l, theta0, Omega0 = symbols('m_0 l_0 G l theta_0 Omega_0', positive=True)

        default_data_dict = {
            self.I: [S.Half * m0 * (l0**2) * no for no in range(1, 3)],
            self.I_1: [S.Half**(no) * (l0**4) for no in range(1, 8)],
            self.I_2: [S.Half**no * (l0**4) for no in range(1, 8)],
            self.l_1: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.l_2: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.theta0: [S.One * self.theta0 * no for no in range(1,5)],
            self.Omega: [S.One * Omega0],
        }

        return default_data_dict

    def max_dynamic_force(self):
        d, theta0= symbols('d theta_0', positive=True)
#         return self.frequency_response_function(
#             self.natural_frequencies()[0]) * self.stiffness_matrix()[0]
        return self.subs({self.theta:theta0}).frequency_response_function() * self.spring_2.stiffness *2/d + self.max_static_force()

class DoubleDiskShaft(ComposedSystem):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertia
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'ddof_undamped_shaft.png'
    real_name = 'ddof_shaft_real.png'
    
    d=Symbol('d', positive=True)
    G=Symbol('G', positive=True)
    
    l=Symbol('l', positive=True)
    l_1=Symbol('l_1', positive=True)
    l_2=Symbol('l_2', positive=True)
    
    I_1=Symbol('I_1', positive=True)
    I_2=Symbol('I_2', positive=True)
    I_m1=Symbol('I_m1', positive=True)
    I_m2=Symbol('I_m2', positive=True)
    
    k_2=Symbol('k_2', positive=True)
    k_1=Symbol('k_1', positive=True)
    
    T_1=Symbol('T_1',positive=True)
    T_2=Symbol('T_2',positive=True)
    
    Omega=Symbol('Omega',positive=True)
 
    theta=dynamicsymbols('theta')
    phi_1=dynamicsymbols('\\varphi_1')                 
    phi_2=dynamicsymbols('\\varphi_2')                 
    phi=dynamicsymbols('\\varphi')

    m0 = Symbol('m_0', positive=True)
    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    T0 = Symbol('T_0', positive=True)

    theta0 = Symbol('theta_0', positive=True)
    Omega =Symbol('Omega', positive=True)

    def __init__(self,
                 l=None,
                 I_m1=None,
                 I_m2=None,
                 k_2=None,
                 k_1=None,
                 T_1=None,
                 T_2=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 phi_1=None,                 
                 phi_2=None,                 
                 phi=None,
                 d=None,
                 theta=None,
                 ivar=Symbol('t'),
                 qs=None,
                 
                 **kwargs):
        
        if  l_1 is not None: self.l_1 = l_1
        if  l_2 is not None: self.l_2 = l_2
        if  I_1 is not None: self.I_1 = I_1
        if  I_2 is not None: self.I_2 = I_2
        if  I_m1 is not None: self.I_m1 = I_m1
        if  I_m2 is not None: self.I_m2 = I_m2
        if T_1 is not None: self.T_1 = T_1
        if T_2 is not None: self.T_2 = T_2
        #if Omega is not None: self.Omega = Omega
        if  k_1 is not None: self.k_1 = k_1
        if  k_2 is not None: self.k_2 = k_2
        if  phi_1 is not None: self.phi_1 = phi_1
        if  phi_2 is not None: self.phi_2 = phi_2 
        if  phi is not None: self.phi = phi 
        if  theta is not None: self.theta = theta 
        #if d is not None: self.d=d    
        self._init_from_components(**kwargs)

    @property
    def components(self):
        components = {}

        self.qs = [self.phi_1,self.phi_2]
        self.k_1 = (self.G*self.I_1)/self.l_1
        self.k_2 = (self.G*self.I_2)/self.l_2

        self.disc_1 = Disk(self.I_m1, pos1=self.phi_1, qs=self.qs)
        self.spring_1 = Spring(self.k_1, self.phi_1, self.phi_2, qs=self.qs)  # left spring
        
        self.disc_2 = Disk(self.I_m2, pos1=self.phi_2, qs=self.qs)
        self.spring_2 = Spring(self.k_2, pos1=self.phi_2, pos2=self.theta,
                               qs=self.qs)  # right spring
        self.moment_disc1=Force(self.T_1, pos1=self.phi_1)
        self.moment_disc2=Force(self.T_2, pos1=self.phi_2)

        components['disc_1'] = self.disc_1
        components['disc_2'] = self.disc_2
        components['spring_1'] = self.spring_1
        components['spring_2'] = self.spring_2
        components['moment_disc1'] = self.moment_disc1
        components['moment_disc2'] = self.moment_disc2

        return components


    def symbols_description(self):
        self.sym_desc_dict = {
            self.I: r'Moment of Inertia',
            self.k_1: r'',
            self.k_2: r'',
        }
        return self.sym_desc_dict

    def get_default_data(self):

        m0, l0 , G, l,  d, T0 = self.m0, self.l0 , self.G, self.l,  self.d, self.T0
        theta0, Omega = self.theta0, self.Omega

        default_data_dict = {
            self.I_m1: [S.Half*m0*(l0**2)*no for no in range(1,9)],
            self.I_m2: [S.Half*m0*(l0**2)*no for no in range(1,9)],
            self.I_1: [S.Half**(no)*(l0**4) for no in range(1,9)],
            self.I_2: [S.Half**no*(l0**4) for no in range(1,9)],
            self.l_1: [S.Half**(no-6)*l0 for no in range(1,9)],
            self.l_2: [S.Half**(no-6)*l0 for no in range(1,9)],
            self.T_1: [T0 * (no) for no in range(1,12)],
            self.T_2: [T0 * (no) for no in range(1,12)],
            self.theta:[theta0* cos(self.Omega * self.ivar) ],
            
        }

        return default_data_dict


    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST
        ]

        return comp_list
    
    def static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        return (free_coeff)
    
    def dynamic_force(self):
        data=self._given_data
        amps = self._fodes_system.steady_solution.as_dict()
        dyn_force=(self.components['spring_2'].force().subs(amps)).subs(data).expand().doit()
        
        return dyn_force
    
    def disc_1_force(self):
        t=self.ivar
        return self.I_m1 * self.steady_solution().diff(t,t)

    def max_static_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[0]/d)
    
    def max_dynamic_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[0]*self.Omega**2

        return  abs(2*(self.I_m1*acc_amp)/d) + self.max_static_disk_1_bearing_force()#.subs(self._given_data)

    def max_static_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[1]/d)
    
    def max_dynamic_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[1]*self.Omega**2

        return  abs(2*(self.I_m2*acc_amp)/d) + self.max_static_disk_2_bearing_force()#.subs(self._given_data)
    
    
    def static_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_1_bearing_force())/(kd*h)
    
    def dynamic_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_1_bearing_force())/(kd*h)
    
    
    def static_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_2_bearing_force())/(kd*h)
    
    def dynamic_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_2_bearing_force())/(kd*h)
    

    
class DoubleDiskShaftHarmonicExcitation(DoubleDiskShaft):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertia
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'ddof_undamped_shaft.png'
    real_name = 'ddof_shaft_real.png'
    
    d=Symbol('d', positive=True)
    G=Symbol('G', positive=True)
    l=Symbol('l', positive=True)
    l_1=Symbol('l_1', positive=True)
    l_2=Symbol('l_2', positive=True)
    I_1=Symbol('I_1', positive=True)
    I_2=Symbol('I_2', positive=True)
    I_m1=Symbol('I_m1', positive=True)
    I_m2=Symbol('I_m2', positive=True)
    k_2=Symbol('k_2', positive=True)
    k_1=Symbol('k_1', positive=True)
    T_1=Symbol('T_1',positive=True)
    T_2=Symbol('T_2',positive=True)
    Omega=Symbol('Omega',positive=True)
#     theta=dynamicsymbols('theta')
    phi_1=dynamicsymbols('\\varphi_1')                 
    phi_2=dynamicsymbols('\\varphi_2')                 
    phi=dynamicsymbols('\\varphi')
    m0 = Symbol('m_0', positive=True)
    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    T0 = Symbol('T_0', positive=True)
    theta = Symbol('theta', positive=True)
    Omega =Symbol('Omega', positive=True)

    def __init__(self,
                 l=None,
                 I_m1=None,
                 I_m2=None,
                 k_2=None,
                 k_1=None,
                 T_1=None,
                 T_2=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 phi_1=None,                 
                 phi_2=None,                 
                 phi=None,
                 d=None,
                 theta=None,
                 Omega=None,
                 ivar=Symbol('t'),
                 qs=None,
                 
                 **kwargs):
        
        if  l_1 is not None: self.l_1 = l_1
        if  l_2 is not None: self.l_2 = l_2
        if  I_1 is not None: self.I_1 = I_1
        if  I_2 is not None: self.I_2 = I_2
        if  I_m1 is not None: self.I_m1 = I_m1
        if  I_m2 is not None: self.I_m2 = I_m2
        if T_1 is not None: self.T_1 = T_1
        if T_2 is not None: self.T_2 = T_2
        if Omega is not None: self.Omega = Omega
        if  k_1 is not None: self.k_1 = k_1
        if  k_2 is not None: self.k_2 = k_2
        if  phi_1 is not None: self.phi_1 = phi_1
        if  phi_2 is not None: self.phi_2 = phi_2 
        if  phi is not None: self.phi = phi 
        if  theta is not None: self.theta = theta
        #if d is not None: self.d=d
        self._init_from_components(**kwargs)
            
    @property
    def components(self):
        components = {}

        self.qs = [self.phi_1,self.phi_2]
        self.k_1 = (self.G*self.I_1)/self.l_1
        self.k_2 = (self.G*self.I_2)/self.l_2

        self.disc_1 = Disk(self.I_m1, pos1=self.phi_1, qs=self.qs)
        self.spring_1 = Spring(self.k_1, self.phi_1, self.phi_2, qs=self.qs)  # left spring
        self.disc_2 = Disk(self.I_m2, pos1=self.phi_2, qs=self.qs)
        self.spring_2 = Spring(self.k_2, pos1=self.phi_2, pos2=self.theta*cos(self.Omega*self.ivar),qs=self.qs)  # right spring
        self.moment_disc1=Force(self.T_1, pos1=self.phi_1)
        self.moment_disc2=Force(self.T_2, pos1=self.phi_2)

        components['disc_1'] = self.disc_1
        components['disc_2'] = self.disc_2
        components['spring_1'] = self.spring_1
        components['spring_2'] = self.spring_2
        components['moment_disc1'] = self.moment_disc1
        components['moment_disc2'] = self.moment_disc2

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.I: r'Moment of Inertia',
            self.k_1: r'',
            self.k_2: r'',
        }
        return self.sym_desc_dict

#     def get_default_data(self):

#         m0, l0 , G, l,  d, T0 = self.m0, self.l0 , self.G, self.l,  self.d, self.T0
#         theta0, Omega0 = symbols('theta_0 Omega_0', positive=True)

#         default_data_dict = {
#             self.I_m1: [S.Half*m0*(l0**2)*no for no in range(1,9)],
#             self.I_m2: [S.Half*m0*(l0**2)*no for no in range(1,9)],
#             self.I_1: [S.Half**(no)*(l0**4) for no in range(1,9)],
#             self.I_2: [S.Half**no*(l0**4) for no in range(1,9)],
#             self.l_1: [S.Half**(no-6)*l0 for no in range(1,9)],
#             self.l_2: [S.Half**(no-6)*l0 for no in range(1,9)],
#             self.T_1: [T0 * (no) for no in range(1,12)],
#             self.T_2: [T0 * (no) for no in range(1,12)],
#             self.theta:[S.One * theta0 * (no) for no in range(1,5)],
# #             self.Omega:[S.One * Omega0],
            
#         }

#         return default_data_dict

    def get_default_data(self):

        m0, l0 , G, l,  d, T0 = self.m0, self.l0 , self.G, self.l,  self.d, self.T0
        theta0, Omega = self.theta0, self.Omega

        default_data_dict = {
            self.k_1:[self.k_1],
            self.k_2:[self.k_2],
            self.I_m1: [S.Half*m0*(l0**2)],
            self.I_m2: [S.Half*m0*(l0**2)],
            self.I_1: [S.Half*(l0**4)],
            self.I_2: [S.Half*(l0**4)],
            self.l_1: [S.Half*l0,2*S.Half*l0,4*S.Half*l0],
            self.l_2: [S.Half*l0,2*S.Half*l0,4*S.Half*l0],
            self.T_1: [T0],
            self.T_2: [T0],
        }

        return default_data_dict

    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST
        ]

        return comp_list
    
    def disc_1_force(self):
        t=self.ivar
        return self.I_m1 * self.steady_solution().diff(t,t)

    def max_static_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[0]/d)
    
    def max_dynamic_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[0]*self.Omega**2

        return  abs(2*(self.I_m1*acc_amp)/d) + self.max_static_disk_1_bearing_force()#.subs(self._given_data)

    def max_static_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[1]/d)
    
    def max_dynamic_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[1]*self.Omega**2

        return  abs(2*(self.I_m2*acc_amp)/d) + self.max_static_disk_2_bearing_force()#.subs(self._given_data)
    
    
    def static_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_1_bearing_force())/(kd*h)
    
    def dynamic_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_1_bearing_force())/(kd*h)
    
    
    def static_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_2_bearing_force())/(kd*h)
    
    def dynamic_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_2_bearing_force())/(kd*h)
    
class DampedDoubleDiskShaftHarmonicExcitation(DoubleDiskShaftHarmonicExcitation):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertia
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'ddof_undamped_shaft.png'
    real_name = 'ddof_shaft_real.png'
    
    d=Symbol('d', positive=True)
    G=Symbol('G', positive=True)
    l=Symbol('l', positive=True)
    l_1=Symbol('l_1', positive=True)
    l_2=Symbol('l_2', positive=True)
    I_1=Symbol('I_1', positive=True)
    I_2=Symbol('I_2', positive=True)
    I_m1=Symbol('I_m1', positive=True)
    I_m2=Symbol('I_m2', positive=True)
    k_2=Symbol('k_2', positive=True)
    k_1=Symbol('k_1', positive=True)
    T_1=Symbol('T_1',positive=True)
    T_2=Symbol('T_2',positive=True)
    Omega=Symbol('Omega',positive=True)
#     theta=dynamicsymbols('theta')
    phi_1=dynamicsymbols('\\varphi_1')                 
    phi_2=dynamicsymbols('\\varphi_2')                 
    phi=dynamicsymbols('\\varphi')
    m0 = Symbol('m_0', positive=True)
    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    T0 = Symbol('T_0', positive=True)
    theta = Symbol('theta', positive=True)
    Omega =Symbol('Omega', positive=True)
    c1=Symbol('c_1',positive=True)
    c2=Symbol('c_2',positive=True)

    def __init__(self,
                 l=None,
                 I_m1=None,
                 I_m2=None,
                 k_2=None,
                 k_1=None,
                 T_1=None,
                 T_2=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 phi_1=None,                 
                 phi_2=None,                 
                 phi=None,
                 d=None,
                 theta=None,
                 Omega=None,
                 ivar=Symbol('t'),
                 qs=None,
                 c1=None,
                 c2=None,
                 
                 **kwargs):
        
        if  l_1 is not None: self.l_1 = l_1
        if  l_2 is not None: self.l_2 = l_2
        if  I_1 is not None: self.I_1 = I_1
        if  I_2 is not None: self.I_2 = I_2
        if  I_m1 is not None: self.I_m1 = I_m1
        if  I_m2 is not None: self.I_m2 = I_m2
        if T_1 is not None: self.T_1 = T_1
        if T_2 is not None: self.T_2 = T_2
        if Omega is not None: self.Omega = Omega
        if  k_1 is not None: self.k_1 = k_1
        if  k_2 is not None: self.k_2 = k_2
        if  phi_1 is not None: self.phi_1 = phi_1
        if  phi_2 is not None: self.phi_2 = phi_2 
        if  phi is not None: self.phi = phi 
        if  theta is not None: self.theta = theta
        if  c2 is not None: self.c2 = c2
        if  c1 is not None: self.c1 = c1
        #if d is not None: self.d=d
        self._init_from_components(**kwargs)
            
    @property
    def components(self):
        components = {}

        self.qs = [self.phi_1,self.phi_2]
        self.k_1 = (self.G*self.I_1)/self.l_1
        self.k_2 = (self.G*self.I_2)/self.l_2

        self.disc_1 = Disk(self.I_m1, pos1=self.phi_1, qs=self.qs)
        self.spring_1 = Spring(self.k_1, self.phi_1, self.phi_2, qs=self.qs)  # left spring
        self.disc_2 = Disk(self.I_m2, pos1=self.phi_2, qs=self.qs)
        self.spring_2 = Spring(self.k_2, pos1=self.phi_2, pos2=self.theta*cos(self.Omega*self.ivar),qs=self.qs)  # right spring
        self.moment_disc1=Force(self.T_1, pos1=self.phi_1)
        self.moment_disc2=Force(self.T_2, pos1=self.phi_2)
        self.damper_1 = Damper(self.c1, pos1=self.phi_1, pos2=self.phi_2, qs=self.qs)
        self.damper_2 = Damper(self.c2, pos1=self.phi_2, pos2=self.phi_2, qs=self.qs)
        

        components['disc_1'] = self.disc_1
        components['disc_2'] = self.disc_2
        components['spring_1'] = self.spring_1
        components['spring_2'] = self.spring_2
        components['moment_disc1'] = self.moment_disc1
        components['moment_disc2'] = self.moment_disc2
        components['damper_1'] = self.damper_1
        components['damper_2'] = self.damper_2

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.I: r'Moment of Inertia',
            self.k_1: r'',
            self.k_2: r'',
        }
        return self.sym_desc_dict


    def get_default_data(self):

        m0, l0 , G, l,  d, T0, lam = self.m0, self.l0 , self.G, self.l,  self.d, self.T0 ,Symbol('lambda')
        theta0, Omega = self.theta0, self.Omega

        default_data_dict = {
            self.I_m1: [S.Half*m0*(l0**2)*no for no in range(1,9)],
            self.I_m2: [S.Half*m0*(l0**2)*no for no in range(1,9)],
            self.I_1: [S.Half**(no)*(l0**4) for no in range(1,9)],
            self.I_2: [S.Half**no*(l0**4) for no in range(1,9)],
            self.l_1: [S.Half**(no-6)*l0 for no in range(1,9)],
            self.l_2: [S.Half**(no-6)*l0 for no in range(1,9)],
            self.T_1: [T0 * (no) for no in range(1,12)],
            self.T_2: [T0 * (no) for no in range(1,12)],
            #self.theta:[theta0* cos(self.Omega * self.ivar) ],
            self.c1: [self.k_1 * (no) * lam for no in range (1,9)],
            self.c2: [self.k_2 * (no) * lam for no in range (1,9)]
            
        }

        return default_data_dict

    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST
        ]

        return comp_list
    
    def disc_1_force(self):
        t=self.ivar
        return self.I_m1 * self.steady_solution().diff(t,t)

    def max_static_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[0]/d)
    
    def max_dynamic_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[0]*self.Omega**2

        return  abs(2*(self.I_m1*acc_amp)/d) + self.max_static_disk_1_bearing_force()#.subs(self._given_data)

    def max_static_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[1]/d)
    
    def max_dynamic_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        acc_amp = self._frf()[1]*self.Omega**2

        return  abs(2*(self.I_m2*acc_amp)/d) + self.max_static_disk_2_bearing_force()#.subs(self._given_data)
    
    
    def static_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_1_bearing_force())/(kd*h)
    
    def dynamic_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_1_bearing_force())/(kd*h)
    
    
    def static_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_2_bearing_force())/(kd*h)
    
    def dynamic_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_2_bearing_force())/(kd*h)
    
    
#TO_DO 158 AF
class TripleShaft(ComposedSystem):
    """
     Triple Shaft system representing a kinematically excited shaft with three disks.
     Describes a physical model of a triple elastic shaft system with three rotating discs subjected to external moments. The class simulates the dynamics of such a system by defining the relevant parameters, dynamics and components.
    Arguments:
    =========
            m = Mass
            - The total mass of the shaft. (Całkowita masa wału.)

        m_1, m_2, m_3 = Masses of the Disks
            - The masses of the first, second, and third disks, respectively. 
              (Masy pierwszego, drugiego i trzeciego dysku.)

        l_1, l_2, l_3 = Lengths of the Shaft Segments
            - The lengths of the shaft segments between the disks. 
              (Długości segmentów wału pomiędzy dyskami.)

        d, d_1, d_2, d_3 = Diameters of the Shaft and Disks
            - The diameters of the shaft and each disk, respectively.
              (Średnica wału i każdego z trzech dysków.)

        G = Kirchhoff module

        M_1, M_2, M_3 = Applied Moments
            - The moments applied to the first, second, and third disks, respectively.
              (Moment siły przyłożony do pierwszego, drugiego i trzeciego dysku.)

        Omega = Angular Frequency
            - The angular frequency of the system's motion. (Częstotliwość kątowa ruchu układu.)

        delta = Phase Shift
            - The phase shift of the oscillatory input. (Przesunięcie fazowe wejściowego sygnału oscylacyjnego.)

        input_displacement = Angular Displacement
            - The angular displacement of the shaft input. (Przemieszczenie kątowe wejścia wału.)

        ivar = Symbol object
            - The independent time variable. (Zmienna niezależna czasu.)

        qs = Dynamicsymbol object
            - Generalized coordinates of the system. (Uogólnione współrzędne układu.)
            


    Example
    =======
   A shaft with three disks oscillating due to an applied moment and connected by springs.
 
    >>> t = symbols('t')
    >>> m, m_1, m_2, m_3 = symbols('m m_1 m_2 m_3')
    >>> l_1, l_2, l_3 = symbols('l_1 l_2 l_3')
    >>> G, Omega, delta = symbols('G Omega delta')
    >>> qs = dynamicsymbols('phi_1, phi_2, phi_3') # Generalized Coordinates
    >>> TripleShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the Triple Shaft system using the TripleShaft class.
    """

    scheme_name = 'MDoFTripleShaft.PNG'
    real_name = 'MDoFTripleShaft_real.jpg'

    # Symbol definitions with default positive values
    m=Symbol('m', positive=True)
    m_1=Symbol('m_1', positive=True)
    m_2=Symbol('m_2', positive=True)
    m_3=Symbol('m_3', positive=True)
    l_1=Symbol('l_1', positive=True)
    l_2=Symbol('l_2', positive=True)
    l_3=Symbol('l_3', positive=True)
    d=Symbol('d', positive=True)
    d_1=Symbol('d_1', positive=True)
    d_2=Symbol('d_2', positive=True)
    d_3=Symbol('d_3', positive=True)
    G = Symbol('G', positive=True)
    M_1=Symbol('M_1',positive=True)
    M_2=Symbol('M_2',positive=True)
    M_3=Symbol('M_3',positive=True)
    Omega=Symbol('Omega',positive=True)
    delta =Symbol('delta', positive=True)
    
    # Dynamicsymbols for generalized coordinates and input displacement
    input_displacement=dynamicsymbols('theta')
    phi_1=dynamicsymbols('\\varphi_1')
    phi_2=dynamicsymbols('\\varphi_2')
    phi_3=dynamicsymbols('\\varphi_3')
    
    phi_r=dynamicsymbols('\\varphi_r')
    phi_l=dynamicsymbols('\\varphi_l')

    def __init__(self,
                 m = symbols('m', positive=True),
                 m_1=Symbol('m_1', positive=True),
                 m_2=Symbol('m_2', positive=True),
                 m_3=Symbol('m_3', positive=True),
                 l_1=Symbol('l_1', positive=True),
                 l_2=Symbol('l_2', positive=True),
                 l_3=Symbol('l_3', positive=True),
                 d=Symbol('d', positive=True),
                 d_1=Symbol('d_1', positive=True),
                 d_2=Symbol('d_2', positive=True),
                 d_3=Symbol('d_3', positive=True),
                 G=Symbol('G', positive=True),
                 M_1=Symbol('M_1', positive=True),
                 M_2=Symbol('M_2', positive=True),
                 M_3=Symbol('M_3', positive=True),
                 Omega=Symbol('\\Omega', positive=True),
                 delta=Symbol('\\delta', positive=True),

                 input_displacement=dynamicsymbols('theta'),
                 phi_1=dynamicsymbols('\\varphi_1'),
                 phi_2=dynamicsymbols('\\varphi_2'),
                 phi_3=dynamicsymbols('\\varphi_3'),

                 phi_l=dynamicsymbols('\\varphi_L'),
                 phi_r=dynamicsymbols('\\varphi_R'),

                 ivar=Symbol('t'),
                 qs=dynamicsymbols('\\varphi_1, \\varphi_2, \\varphi_3,'),
                 **kwargs):

        # Initializing class attributes with provided arguments or default values
        if  m is not None: self.m = m
        if  m_1 is not None: self.m_1 = m_1
        if  m_2 is not None: self.m_2 = m_2
        if  m_3 is not None: self.m_3 = m_3
        if  l_1 is not None: self.l_1 = l_1
        if  l_2 is not None: self.l_2 = l_2
        if  l_3 is not None: self.l_3 = l_3
        if  d is not None: self.d = d
        if  d_1 is not None: self.d_1 = d_1
        if  d_2 is not None: self.d_2 = d_2
        if  d_3 is not None: self.d_3 = d_3
        if  G is not None: self.G = G
        if  M_1 is not None: self.M_1 = M_1
        if  M_2 is not None: self.M_2 = M_2
        if  M_3 is not None: self.M_3 = M_3
        if Omega is not None: self.Omega = Omega
        if delta is not None: self.delta = delta
        if  phi_1 is not None: self.phi_1 = phi_1
        if  phi_2 is not None: self.phi_2 = phi_2
        if  phi_3 is not None: self.phi_3 = phi_3
        if  phi_l is not None: self.phi_l = phi_l
        if  phi_r is not None: self.phi_r = phi_r
        if  input_displacement is not None: self.input_displacement = input_displacement

        self.qs = [self.phi_1,self.phi_2,self.phi_3]
               
        self.theta = input_displacement
        self._init_from_components(**kwargs)

        
        


    @property
    def components(self):
        components = {}

        # Moment of inertia calculations
        I_0=S.Half*pi*(self.d/2)**4
        I_1=S.Half*self.m_1*(self.d_1/2)**2
        I_2=S.Half*self.m_2*(self.d_2/2)**2
        I_3=S.Half*self.m_3*(self.d_3/2)**2
        
        # Assigning calculated moments of inertia to class attributes
        self.I_m0 = I_0
        self.I_m1 = I_1
        self.I_m2 = I_2
        self.I_m3 = I_3
        
        # Spring coefficients
        self.k_1=(self.G*I_0)/self.l_1
        self.k_2=(self.G*I_0)/self.l_2
        self.k_3=(self.G*I_0)/self.l_3

        # Defining system components
        self.disc_1 = Disk(self.I_m1, pos1=self.phi_1, qs=self.qs)
        self.spring_1 = Spring(self.k_1, self.phi_1, self.phi_2, qs=self.qs)  # left spring
        
        self.disc_2 = Disk(self.I_m2, pos1=self.phi_2, qs=self.qs)
        self.spring_2 = Spring(self.k_2, pos1=self.phi_2, pos2=self.theta,qs=self.qs)  # right spring
               
        self.disc_3 = Disk(self.I_m3, pos1=self.phi_3, qs=self.qs)
        self.spring_3 = Spring(self.k_3, pos1=self.phi_3, pos2=self.theta,qs=self.qs)
               
        self.moment_disc1=Force(self.M_1, pos1=self.phi_1)
        self.moment_disc2=Force(self.M_2, pos1=self.phi_2)
        self.moment_disc3=Force(self.M_3, pos1=self.phi_3)

        components['disc_1'] = self.disc_1
        components['disc_2'] = self.disc_2
        components['disc_3'] = self.disc_3
        components['spring_1'] = self.spring_1
        components['spring_2'] = self.spring_2
        components['spring_3'] = self.spring_3
        components['moment_disc1'] = self.moment_disc1
        components['moment_disc2'] = self.moment_disc2
        components['moment_disc3'] = self.moment_disc3

        return components
#     def symbols_description(self):
#         self.sym_desc_dict = {
#             self.I: r'Moment of Inertia',
#             self.k_1: r'',
#             self.k_2: r'',
#         }
#         return self.sym_desc_dict
    def get_numerical_data(self):

        default_data_dict = {

            
            
            self.m_1: [1],
            self.m_2: [1],
            self.m_3: [1],

            self.l_1: [1],
            self.l_2: [1],
            self.l_3: [2],

            self.d_1: [1],
            self.d_2: [1],
            self.d_3: [1],
            self.d: [1],

            self.G:[1],
            
            self.delta:[0],

            self.input_displacement:[0],
            
            self.theta:[0],
            
            self.M_1:[1],
            self.M_2:[1],
            self.M_3:[1],
        }
        return default_data_dict
    def get_default_data(self):


        m0, M0, l0 , d0 ,M0= symbols('m_0 M_0 l_0 d_0 M_0', positive=True)
        theta0, Omega = symbols('theta_0 Omega', positive=True)

        default_data_dict = {
            
            Symbol('I_m'):[S.Half*Symbol('m')*Symbol('R')**2],
            Symbol('I_S'):[S.Half*pi*Symbol('R_S')**4],
            Symbol('k_S'):[Symbol('G')*Symbol('I_S')/Symbol('L_S')],
            
            
            self.m_1: [S.Half * m0, 1 * m0, 2 * m0, 1 * m0, S.Half * m0],
            self.m_2: [1 * m0, 2 * m0, S.Half * m0, 1 * m0, 2 * m0],
            self.m_3: [1 * m0, 2 * m0, S.Half * m0, 1 * m0, 2 * m0],

            self.l_1: [1 * l0, 2 * l0, S.Half * l0, 2 * l0, S.Half * l0],
            self.l_2: [1 * l0, 2 * l0, S.Half * l0, 2 * l0, S.Half * l0],
            self.l_3: [2 * l0, 4 * l0, S.Half * l0, 2 * l0, S.Half * l0],

            self.d_1: [1 * d0, 2 * d0, S.Half * d0, 2 * d0, S.Half * d0],
            self.d_2: [1 * d0, 2 * d0, S.Half * d0, 2 * d0, S.Half * d0],
            self.d_3: [2 * d0, 4 * d0, S.Half * d0, 2 * d0, S.Half * d0],
            self.d: [2 * d0, 4 * d0, S.Half * d0, 2 * d0, S.Half * d0],

#             self.phi_1:[self.phi_l,0],
#             self.phi_2:[self.phi_l,self.phi_r],
#             self.phi_3:[self.phi_r,0],
            
            self.delta:[0],

            self.input_displacement:[0],
            
            self.theta:[0],
            
            self.M_1:[M0],
            self.M_2:[M0],
            self.M_3:[M0],
            

        }

        return default_data_dict


    def get_random_parameters(self):

        default_data_dict = self.get_default_data()

        parameters_dict = {
            key: random.choice(items_list)
            for key, items_list in default_data_dict.items()
        }

        if parameters_dict[self.phi_1] != self.phi_l or parameters_dict[self.phi_2] != self.phi_l:

            parameters_dict[self.phi_1] = self.phi_l

        if parameters_dict[self.phi_2] != self.phi_r or parameters_dict[self.phi_3] != self.phi_r:

            parameters_dict[self.phi_3] = self.phi_r
            

        return parameters_dict
    
    def max_static_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[0]/d)
    
    def max_dynamic_disk_1_bearing_force(self):
        d=Symbol('d',positive=True)

        
        frf = self._frf()
        acc_amp = (self.phi_1).subs(self._given_data).subs({self.phi_l:frf[0],self.phi_r:frf[1]})
        

        return  abs(2*(self.I_m1*acc_amp)/d) + self.max_static_disk_1_bearing_force()#.subs(self._given_data)

    def max_static_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)
        return abs(2*self.static_load().doit()[1]/d)
    
    def max_dynamic_disk_2_bearing_force(self):
        d=Symbol('d',positive=True)

        
        frf = self._frf()
        acc_amp = (self.phi_2).subs(self._given_data).subs({self.phi_l:frf[0],self.phi_r:frf[1]})

        return  abs(2*(self.I_m2*acc_amp)/d) + self.max_static_disk_2_bearing_force()#.subs(self._given_data)
    
    
    def static_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_1_bearing_force())/(kd*h)
    
    def dynamic_disk_1_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_1_bearing_force())/(kd*h)
    
    
    def static_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_static_disk_2_bearing_force())/(kd*h)
    
    def dynamic_disk_2_key_length(self):
        kd=Symbol('k_d', positive=True)
        h=Symbol('h', positive=True)
        return (2*self.max_dynamic_disk_2_bearing_force())/(kd*h)
    
    def unit_dict(self):
        units_dict = {
            self.m: ureg.kilogram,  # Mass of the shaft
            self.m_1: ureg.kilogram,  # Mass of the first disk
            self.m_2: ureg.kilogram,  # Mass of the second disk
            self.m_3: ureg.kilogram,  # Mass of the third disk
            self.l_1: ureg.meter,  # Length of the first shaft segment
            self.l_2: ureg.meter,  # Length of the second shaft segment
            self.l_3: ureg.meter,  # Length of the third shaft segment
            self.d: ureg.meter,  # Diameter of the shaft
            self.d_1: ureg.meter,  # Diameter of the first disk
            self.d_2: ureg.meter,  # Diameter of the second disk
            self.d_3: ureg.meter,  # Diameter of the third disk
            self.G: ureg.pascal,  # Shear modulus (Moduł Kirchhoffa)
            self.M_1: ureg.Nm,  # Moment applied on the first disk (torque)
            self.M_2: ureg.Nm,  # Moment applied on the second disk (torque)
            self.M_3: ureg.Nm,  # Moment applied on the third disk (torque)
            self.Omega: ureg.radian / ureg.second,  # Angular velocity (rad/s)
            self.delta: ureg.radian,  # Phase difference (radians)
            self.input_displacement: ureg.radian,  # Input angular displacement (radians)
            self.phi_1: ureg.radian,  # Angular displacement of the first disk (radians)
            self.phi_2: ureg.radian,  # Angular displacement of the second disk (radians)
            self.phi_3: ureg.radian,  # Angular displacement of the third disk (radians)
            self.phi_r: ureg.radian,  # Angular displacement on the right side (radians)
            self.phi_l: ureg.radian,  # Angular displacement on the left side (radians)
            self.phi_1.diff(self.ivar): ureg.radian / ureg.second,  # Angular velocity of the first disk (rad/s)
            self.phi_1.diff(self.ivar, 2): ureg.radian / ureg.second**2,  # Angular acceleration of the first disk (rad/s**2)
            self.phi_2.diff(self.ivar): ureg.radian / ureg.second,  # Angular velocity of the second disk (rad/s)
            self.phi_2.diff(self.ivar, 2): ureg.radian / ureg.second**2,# Angular acceleration of the second disk (rad/s**2)
            self.phi_3.diff(self.ivar): ureg.radian / ureg.second,  # Angular velocity of the third disk (rad/s)
            self.phi_3.diff(self.ivar, 2): ureg.radian / ureg.second**2,# Angular acceleration of the third disk (rad/s**2)
            self.ivar: ureg.second,  # Time (seconds)
            }
        
        return units_dict

#Grześ
class SDoFDampedShaft(ComposedSystem):
    """Ready to use sample Double Degree of Freedom System represents the Kinematicly excited shaft with two disks.
    =========
            I = Moment of Inertia
                -Moment of Inertia in case of both disc

            k_1 =Right spring coefficient
                -Right spring carrying the system

            k_2 =Left spring coefficient
                -Left spring carrying the system

            ivar = symbol object
                -Independant time variable

            qs = dynamicsymbol object
                -Generalized coordinates

    Example
    =======
    A mass oscillating up and down while being held up by a spring with a spring constant k

    >>> t = symbols('t')
    >>> I, k1, k2 = symbols('I, k_1, k_2')
    >>> qs = dynamicsymbols('phi_1, phi_2') # Generalized Coordinates
    >>> DDoFShaft()

    -defines the symbols and dynamicsymbols
    -finally determines the instance of the system using class DDoFShaft
    """

    scheme_name = 'damped_shaft_phi.png'
    real_name = 'ddof_shaft_real.png'
    detail_scheme_name = 'parallel_key_load.png'
    detail_real_name = 'shaft_with_key.png'

    l0 = Symbol('l_0', positive=True)
    G = Symbol('G', positive=True)
    I = Symbol('I', positive=True)
    l_1 = Symbol('l_1', positive=True)
    l_2 = Symbol('l_2', positive=True)
    I_1 = Symbol('I_1', positive=True)
    I_2 = Symbol('I_2', positive=True)
    c_1 = Symbol('c_1', positive=True)
    c_2 = Symbol('c_2', positive=True)
    k_1 = Symbol('k_1', positive=True)
    k_2 = Symbol('k_2', positive=True)
    Ms = Symbol('M_s', positive=True)
    Omega = Symbol('Omega', positive=True)

    theta = dynamicsymbols('theta')
    phi = dynamicsymbols('\\varphi')

    def __init__(self,
                 l0=None,
                 G=None,
                 I=None,
                 l_1=None,
                 l_2=None,
                 I_1=None,
                 I_2=None,
                 c_1=None,
                 c_2=None,
                 k_1=None,
                 k_2=None,
                 Ms=None,
                 phi=None,
                 theta=None,
                 ivar=Symbol('t'),
                 qs=None,
                 **kwargs):
        
        if G is not None: self.G = G
        if I is not None: self.I = I
        if Ms is not None: self.Ms = Ms
        #if Omega is not None: self.Omega = Omega
        if l_1 is not None: self.l_1 = l_1
        if l_2 is not None: self.l_2 = l_2
        if I_1 is not None: self.I_1 = I_1
        if I_2 is not None: self.I_2 = I_2
        if c_1 is not None: self.c_1 = c_1
        if c_2 is not None: self.c_2 = c_2
        if k_1 is not None: self.k_1 = k_1
        if k_2 is not None: self.k_2 = k_2
        if phi is not None: self.phi = phi
        if theta is not None: self.theta = theta

        theta = self.theta
        self.qs = [self.phi]
        self.ivar = ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components={}

        self.disc_1 = Disk(self.I, pos1=self.phi, qs=self.qs)
        self.spring_1 = Spring(self.k_1 * self.k_2 / (self.k_2 + self.k_1), pos1=self.phi, pos2=self.theta, qs=self.qs)
        self.moment = Force(self.Ms, pos1=self.phi, qs=self.qs)
        self.damper_1 = Damper(self.c_1 * self.c_2/(self.c_2 + self.c_1), pos1=self.phi, pos2=self.theta, qs=self.qs)

        components['_disc_1'] = self.disc_1
        components['_spring_1'] = self.spring_1
        components['_moment'] = self.moment
        components['_damper_1']=self.damper_1

        return components

    def symbols_description(self):
        self.sym_desc_dict = {
            self.I: r'Moment of Inertia',
            self.k_1: r'',
            self.k_2: r'',
        }
        return self.sym_desc_dict

    def get_default_data(self):

        m0, l0, G, l, lamb = symbols('m_0 l_0 G l lambda', positive=True)
        theta0, Omega = symbols('theta_0, Omega', positive=True)

        default_data_dict = {
            self.c_1: [lamb * self.k_1],
            self.c_2: [lamb * self.k_2],
            self.k_1: [(self.G * self.I_1) / self.l_1],
            self.k_2: [(self.G * self.I_2) / self.l_2],
            self.I: [S.Half * m0 * (l0**2) * no for no in range(1, 3)],
            self.I_1: [S.Half**(no) * (l0**4) for no in range(1, 8)],
            self.I_2: [S.Half**no * (l0**4) for no in range(1, 8)],
            self.l_1: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.l_2: [S.Half**(no - 6) * l0 for no in range(1, 8)],
            self.theta: [theta0 * cos(Omega * self.ivar)],
        }

        return default_data_dict

    def disc_force(self):
        t = self.ivar
        return self.I * self.steady_solution().diff(t, t)

    def max_static_force_pin(self):
        d = Symbol('d', positive=True)
        return 2 * self.Ms / d

    def max_dynamic_force_pin(self):
        d = Symbol('d', positive=True)
        return self.frequency_response_function(
            self.natural_frequencies()[0]) * self.stiffness_matrix()[0]

    def max_static_bearing_force(self):
        d = Symbol('d', positive=True)
        return abs(2 * self.static_load()[0] / d)

    def max_dynamic_bearing_force(self):
        d = Symbol('d', positive=True)
        acc_amp = self.frequency_response_function() * self.Omega**2

        return abs(
            2 * (self.I * acc_amp) /
            d) + self.max_static_bearing_force()  #.subs(self._given_data)

    def static_key_length(self):
        kd = Symbol('k_d', positive=True)
        h = Symbol('h', positive=True)
        return (2 * self.max_static_bearing_force()) / (kd * h)

    def dynamic_key_length(self):
        kd = Symbol('k_d', positive=True)
        h = Symbol('h', positive=True)
        return (2 * self.max_dynamic_bearing_force()) / (kd * h)



from numpy import positive
from sympy import (flatten, SeqFormula, Function, Symbol, symbols, Eq, Matrix,
                   S, oo, dsolve, solve, Number, pi, cos, sin, Tuple,
                   Derivative,sqrt)

from sympy.physics.vector.printing import vpprint, vlatex


class ContinuousSystem:
    

    
    
    def __init__(self,
                 L,
                 q,
                 bc_dict=None,
                 t_var=Symbol('t'),
                 spatial_var=Symbol('x'),
                 derivative_order=2,
                 label=None,
                 system=None,
                 time_comp=Function('T')(Symbol('t')),
                 spatial_comp=Function('X')(Symbol('x')),
                 mode_symbol=Symbol('k',positive=True),
                 **kwargs):

        if system:
            t_var = system.t
            L = system.L
            q = system.q
            spatial_var = system.r
            derivative_order = system.diff_ord
            bc_dict = system.bc_dict
            time_comp = system.time_comp
            spatial_comp = system.spatial_comp
            mode_symbol = system._mode_symbol



        self.t = t_var
        self.L = L
        self.q = q
        self.r = flatten((spatial_var, ))
        self.diff_ord = derivative_order
        self.bc_dict = bc_dict
        self._bc = Symbol('BC')
        self._mode_symbol = mode_symbol

        self.time_comp=time_comp
        self.spatial_comp=spatial_comp

        self._sep_expr = Symbol('k', positive=True)

        if label == None:
            label = self.__class__.__name__ + ' on ' + str(self.q)

        self._label = label
        self._given_data={}
        
        self._nonlinear_base_system=None
        

    @property
    def BC(self):
        return self._bc

    def __call__(self, *args, label=None):
        """
        Returns the label of the object or class instance with reduced Degrees of Freedom.
        """

        if isinstance(args[0], str):
            if label:
                self._label = label
            else:
                self._label = args[0]

            return self

    def __str__(self):

        return self._label

    def __repr__(self):

        return self.__str__()

    def subs(self, *args, **kwargs):
        
        given_data=args[0]
        
        L_new = self.L.subs(*args)
        q_new = self.q.subs(*args)

        bc_trap = self.BC.subs(*args)

        if bc_trap == self.BC:
            bc_trap = self.bc_dict

        new_system = ContinuousSystem(L=L_new,
                                      q=q_new,
                                      bc_dict=bc_trap,
                                      t_var=self.t,
                                      spatial_var=self.r,
                                      derivative_order=self.diff_ord,
                                      label=self._label)
        
        new_sys = type(self)(L_new, q_new, system=new_system)
        new_sys._given_data=given_data
        new_sys._nonlinear_base_system = self._nonlinear_base_system

        return new_sys

    @property
    def _eoms(self):
        return self.governing_equation

    def inertia_force(self):
        q = self.q
        t = self.t
        L = self.L

        return L.diff(q.diff(t)).diff(t)

    def restoring_force(self):
        q = self.q
        t = self.t
        L = self.L

        return sum([(-1)**(order + 2) *
                    L.diff(q.diff(r_var, order + 1)).diff(r_var, order + 1)
                    for r_var in self.r
                    for order in range(self.diff_ord)]) - L.diff(q)

    def governing_equation(self):

        return self.inertia_force() + self.restoring_force()

    def eom_coeff(self, expr):

        return self.governing_equation().coeff(expr)

    def apply_separation(self,
                         time_comp=Function('T')(Symbol('t')),
                         spatial_comp=Function('X')(Symbol('x'))):

        return self.governing_equation().subs(self.q,
                                              time_comp * spatial_comp).doit()

    def separated_vars_eqn(self,
                           time_comp=Function('T')(Symbol('t')),
                           spatial_comp=Function('X')(Symbol('x'))):

        eqn_with_subs = self.apply_separation(time_comp, spatial_comp)

        return Eq((eqn_with_subs.coeff(spatial_comp) / (time_comp)),
                  -eqn_with_subs.coeff(time_comp) / spatial_comp)

    def spatial_eqn(self,
                    sep_expr=None,
                    spatial_comp=Function('X')(Symbol('x'))):

        if not sep_expr:
            sep_expr = self._sep_expr

        separated_eqn_rhs = self.separated_vars_eqn(
            spatial_comp=spatial_comp).rhs

        return Eq(separated_eqn_rhs, sep_expr)

    def time_eqn(self, sep_expr=None, time_comp=Function('T')(Symbol('t'))):

        if not sep_expr:
            sep_expr = self._sep_expr

        separated_eqn_lhs = self.separated_vars_eqn(time_comp=time_comp).lhs

        return Eq(separated_eqn_lhs, sep_expr)

    def spatial_general_solution(self,
                                 sep_expr=None,
                                 spatial_comp=Function('X')(Symbol('x'))):

        if not sep_expr:
            sep_expr = self._sep_expr

        spatial_ode = self.spatial_eqn(sep_expr, spatial_comp)

        return dsolve(spatial_ode,
                      spatial_comp)  #.rewrite(cos).expand().simplify()

    def fundamental_matrix(self,
                           bc_dict=None,
                           sep_expr=None,
                           spatial_comp=Function('X')(Symbol('x'))):

        if not sep_expr:
            sep_expr = self._sep_expr

        if bc_dict:
            self.bc_dict = bc_dict
        else:
            bc_dict = self.bc_dict

        spatial_sol = self.spatial_general_solution(sep_expr=sep_expr,
                                                    spatial_comp=spatial_comp)

        fun_eqns = Matrix([
            spatial_sol.rhs.subs(spatial_comp.args[0],
                                 flatten([key.args[-1]])[0]) - val
            for key, val in bc_dict.items()
        ])

        matrix_comps_list = []

        for key, val in bc_dict.items():

            #display(list(key.atoms(Symbol,Number) - spatial_comp.atoms(Symbol)))
            free_sym = list(
                key.atoms(Symbol, Number) - spatial_comp.atoms(Symbol))[-1]
            #print({spatial_sol.lhs:spatial_sol.rhs, spatial_sol.lhs.subs(spatial_comp.args[0],free_sym):spatial_sol.rhs, })
            matrix_comps_list += [(key.subs({
                spatial_sol.lhs:
                spatial_sol.rhs,
                spatial_sol.lhs.subs(spatial_comp.args[0], free_sym):
                spatial_sol.rhs.subs(spatial_comp.args[0], free_sym),
            }).doit() - val)]

        fun_eqns = Matrix(matrix_comps_list)

        return fun_eqns.jacobian(symbols('C1:' + str(len(bc_dict) + 1)))

    def char_poly(self,
                  bc_dict=None,
                  sep_expr=None,
                  spatial_comp=Function('X')(Symbol('x'))):

        if not sep_expr:
            sep_expr = self._sep_expr

        if bc_dict:
            self.bc_dict = bc_dict
        else:
            bc_dict = self.bc_dict

        return self.fundamental_matrix(bc_dict, sep_expr,
                                       spatial_comp).det().simplify()


#     def eigenvalues(self, bc_dict=None, sep_expr=Symbol('k',positive=True), arg=Symbol('k',positive=True), spatial_comp=Function('X')(Symbol('x')), index=Symbol('n', integer=True, positive=True)):

#         if bc_dict:
#             self.bc_dict=bc_dict
#         else:
#             bc_dict=self.bc_dict

#         root = solve(self.char_poly(bc_dict, sep_expr, spatial_comp), arg)[0]

#         spatial_span = list(root.free_symbols)[0]

#         return SeqFormula(root+(index-1)/spatial_span*pi, (index, 0, oo))

    def eigenvalues(self,
                    bc_dict=None,
                    sep_expr=None,
                    arg=None,
                    spatial_comp=Function('X')(Symbol('x')),
                    index=Symbol('n', integer=True, positive=True)):

        if not sep_expr:
            sep_expr = self._sep_expr

        if bc_dict:
            self.bc_dict = bc_dict
        else:
            bc_dict = self.bc_dict

        if arg is None:
            arg = self._mode_symbol


        roots = solve(self.char_poly(bc_dict, sep_expr, spatial_comp), arg)
#         print('+++++++++++++ roots from eigs')
#         print(roots)

        if len(roots) == 1:
            spatial_span = roots[0]
            first_root=roots[0]
        else:
            spatial_span = roots[1] - roots[0]
            first_root=roots[0]
            
        seq=SeqFormula(first_root + (index -1) * spatial_span,
                          (index, 1, oo))

#         display(seq)
        
        return seq

    def eigenmodes(self,
                   mode_no,
                   bc_dict=None,
                   sep_expr=None,
                   arg=None,
                   spatial_comp=Function('X')(Symbol('x')),
                   index=Symbol('n', integer=True, positive=True)):

        if arg is None:
            arg = self._mode_symbol

        if not sep_expr:
            sep_expr = self._sep_expr

        if bc_dict:
            self.bc_dict = bc_dict
        else:
            bc_dict = self.bc_dict

        C_list = list(symbols('C1:' + str(len(bc_dict) + 1)))

        eig_value = self.eigenvalues(bc_dict, sep_expr, arg, spatial_comp,
                                     index).formula.expand().simplify()

        mode_eqn = self.fundamental_matrix(bc_dict, sep_expr,
                                           spatial_comp) * Matrix(C_list)
        #display( eig_value)
        #display( eig_value)

        eig_aid = ({
            comp: 0
            for comp in (mode_eqn.atoms(sin, cos))
            if comp.subs(arg, eig_value).subs(index, mode_no).n() < 0.01
        })

#         print('mode_eqn')
#         display(mode_eqn.subs(eig_aid))
#         display(mode_eqn)
#         print('aid')
#         display(eig_aid)

        mode_subs = solve(
            mode_eqn.applyfunc(lambda x: x.subs(eig_aid)), C_list)
        #         mode_subs = solve(mode_eqn[1:], C_list)

#         print('solve for Cs')
#         display(mode_subs)

        

        gen_sol=self.spatial_general_solution(sep_expr=sep_expr, spatial_comp=spatial_comp).rhs
#         display(gen_sol)
        
        gen_sol=gen_sol.subs(arg, eig_value).subs(eig_aid)
#         display(gen_sol)
        
        return gen_sol.subs(mode_subs).subs(arg, eig_value).subs({c_var: 1 for c_var in C_list}).subs(index, mode_no).n(2)

    def phase_velocity(self):


        X_fun=self.spatial_comp
        T_fun=self.time_comp

        nat_freq=Symbol('omega',positive=True)

        #display(self.time_eqn())

        nat_freq_eq=self.time_eqn().subs({T_fun.diff(self.t,2) :-T_fun*nat_freq **2 })
        
        nat_freq_expr = solve( nat_freq_eq.lhs - nat_freq_eq.rhs, nat_freq **2 )

        return sqrt(max(nat_freq_expr))/self._mode_symbol


class PlaneStressProblem:
    def __init__(self,
                 #disp_func=[Function('\\mathit{u}')(Symbol('r')), 0],
                 disp_func,
                 stress_tensor=Matrix(2, 2, [
                     Function('\\sigma_r')(Symbol('r')), 0, 0,
                     Function('\\sigma_\\varphi')(Symbol('r'))
                 ]),
                 bc_dict=None,
                 coords=[Symbol('r'), Symbol('\\varphi')],
                 E=Symbol('E', positive=True),
                 nu=Symbol('\\nu', positive=True),
                 D=Symbol('D', positive=True),                 
                 label=None,
                 system=None,
                 volumetric_load=None,
                 **kwargs):

        if system:
            disp_func = system.u
            stress_tensor = system.stress
            coords = system.coords
            E = system.E_module
            nu = system.poisson
            D=system.D
            bc_dict = system.bc_dict
            volumetric_load=system.volumetric_load
            
#         print('__init')
#         print(type(self))
#         display(disp_func)

        self.u = Matrix(disp_func)
        
#         print('__init')
#         print(type(self))
#         display(self.u)
        
        
        self.stress = stress_tensor
        self.E_module = E
        self.poisson = nu
        self.D = D
        self.coords = coords
        self.dim = max(self.u.shape)
        self._equilibrium_eqn = None
        self._integrantion_cs = None
        self._bc = Symbol('BC')
        self.bc_dict = bc_dict
        self.volumetric_load = volumetric_load
        #print('__init__',self.bc_dict)
        if label == None:
            label = self.__class__.__name__ + ' on ' + str(self.coords)

        self._label = label
        self._subs_dict = {}
        self._given_data={}

    @property
    def BC(self):
        return self._bc

    def __call__(self, *args, label=None):
        """
        Returns the label of the object or class instance with reduced Degrees of Freedom.
        """

        if isinstance(args[0], str):
            if label:
                self._label = label
            else:
                self._label = args[0]

            return self

    def __str__(self):

        return self._label

    def __repr__(self):

        return self.__str__()

    def subs(self, *args, **kwargs):
        
#         print('u_old')
#         display(self.u)
        given_data=args[0]
    
        E_new, nu_new, u_new, stress_new, vol_load_new, D_new = Tuple(
            self.E_module, self.poisson, self.u, self.stress,
            self.volumetric_load,self.D).subs(*args)


        bc_trap = self.BC.subs(*args)

        if bc_trap == self.BC:
            bc_trap = self.bc_dict


        new_system = type(self)(disp_func=u_new,
                                        stress_tensor=stress_new,
                                        bc_dict=bc_trap,
                                        volumetric_load=vol_load_new,
                                        coords=self.coords,
                                        E=E_new,
                                        nu=nu_new,
                                        D=D_new,
                                        label=self._label,
                                        system=None,
                                        **kwargs)

        
        new_sys = type(self)(system=new_system)
        new_sys._given_data=given_data
        
        return new_sys

    @property
    def _eoms(self):
        return self.governing_equation

    def strain_field(self, disp_func=None):
        if disp_func is not None:
            self.u = disp_func
            self.dim = max(self.u.shape)

        self.strain = Matrix(self.dim, self.dim, [
            Derivative(self.u[0], self.coords[0]), 0, 0,
            self.u[0] / self.coords[0]
        ])
        return self.strain

    def stress_field(self, stress_tensor=None):
        if stress_tensor is not None:
            self.stress = stress_tensor

        return self.stress

    def constitutive_equation(self, E=None, nu=None):
        if E is not None:
            self.E_module = E

        if nu is not None:
            self.poisson = nu

        return Matrix(self.dim, self.dim, [
            1 / self.E_module, -self.poisson / self.E_module,
            -self.poisson / self.E_module, 1 / self.E_module
        ])

    def strain_load_equation(self):

        return self.constitutive_equation() * Matrix(
            [self.stress[0, 0], self.stress[-1, -1]])

    def load_strain_equation(self, dict=None):

        load_mat = self.constitutive_equation().inv() * Matrix(
            [self.strain_field()[0, 0],
             self.strain_field()[-1, -1]])

        if dict is True:
            return {
                self.stress[0, 0]: load_mat[0],
                self.stress[-1, -1]: load_mat[-1]
            }
        else:
            return load_mat

    def load_equilibrium(self, volumetric_load=None):

#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        return Derivative(
            self.stress[0] * self.coords[0], self.coords[0]
        ) - self.stress[-1] + self.volumetric_load * self.coords[0]

    def equilibrium_eqn(self, volumetric_load=None, subs_dict=None):

        if subs_dict:
            self._subs_dict = subs_dict

#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        stress_r, stress_phi = list(self.load_strain_equation())

        
        
        self._equilibrium_eqn = self.load_equilibrium(
            volumetric_load=self.volumetric_load).doit().subs(
                self.load_strain_equation(dict=True)).subs(
                    self._subs_dict).expand().simplify()

        return self._equilibrium_eqn

    def solution(self, volumetric_load, dvar, ics=None, ivar=Symbol('r')):


        #         display(ics)
#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        
            
        if not self._equilibrium_eqn:
            self._equilibrium_eqn=self.equilibrium_eqn( volumetric_load=self.volumetric_load, subs_dict=self._subs_dict)

        C_list=symbols('C1:' + str(self.dim + 1))
            
        if ics is None:
            #display(self._equilibrium_eqn)
            
            return (Eq(dvar,-(ivar*(self._equilibrium_eqn.subs(dvar,0)/ivar).doit().integrate(ivar)).integrate(ivar)/self.D/ivar  + C_list[0]/ivar + C_list[1]*ivar  )  )
            
            #return dsolve(self._equilibrium_eqn, dvar)
        else:
            return dsolve(self._equilibrium_eqn, dvar, ics=ics)

    def integration_constants(self,
                              volumetric_load,
                              dvar,
                              ics,
                              ivar=Symbol('r')):
        
#         display('vol',self.volumetric_load)

#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        sol = self.solution(volumetric_load=self.volumetric_load, dvar=dvar)

        conds_eqns = [
            ((key.subs(sol.lhs, sol.rhs).doit()) - cond_value).doit()
            for key, cond_value in ics.items()
        ]

#         display(conds_eqns)
        
        self._integrantion_cs = solve(conds_eqns,
                                      symbols('C1:' + str(self.dim + 1)))

        return self._integrantion_cs

    def particular_solution(self, volumetric_load, dvar, ics,
                            ivar=Symbol('r')):

#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        sol = self.solution(volumetric_load=self.volumetric_load, dvar=dvar)

        if not self._integrantion_cs:
            self._integrantion_cs=self.integration_constants(
                              volumetric_load=self.volumetric_load,
                              dvar=dvar,
                              ics=ics,
                              ivar=ivar)
        
        return sol.subs(self._integrantion_cs)

    def loads(self, volumetric_load, dvar, ics, ivar=Symbol('r')):

#         if volumetric_load:
#             self.volumetric_load = volumetric_load

        particular_sol = self.particular_solution(self.volumetric_load,
                                                  dvar,
                                                  ics,
                                                  ivar=Symbol('r'))

        return self.load_strain_equation().subs(particular_sol.lhs,
                                                particular_sol.rhs)



#96 DONE

from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from  ..continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST

class MissileTrajectoryAirless(ComposedSystem):
    
    m=Symbol('m',positive=True)
    g=Symbol('g',positive=True)
    c=Symbol('c',positive=True)
    x=dynamicsymbols('x',positive=True)
    y=dynamicsymbols('y',positivie=True)
    c0=Symbol('C0',positive=True)
    ivar=Symbol('t')
    
    def __init__(self,
                 m=None,
                 g=None,
                 c=None,
                 x=None,
                 y=None,
                 ivar=None,
                 **kwargs):
        
        if m is not None: self.m = m
        if g is not None: self.g = g
        if c is not None: self.c = c
        if x is not None: self.x = x
        if y is not None: self.y = y
        if ivar is not None: self.ivar= ivar

        self.qs = [self.x,self.y]
        
        self._init_from_components(**kwargs)
       
    @property
    def components(self):

        v_x=(self.x).diff(self.ivar)
        v_y=(self.y).diff(self.ivar)
        components = {}

        self._mass_x = MaterialPoint(self.m,pos1=self.x,qs=self.qs)
        self._mass_y = MaterialPoint(self.m,pos1=self.y,qs=self.qs)

        self._gravity = GravitationalForce(self.m,self.g,pos1=self.y,qs=self.qs)
        
        

        
        components['_mass_x'] = self._mass_x
        components['_mass_y'] = self._mass_y
        components['_gravity'] = self._gravity
        #components['_drag_x'] = self._drag_x
        #components['_drag_y'] = self._drag_y
        return components

#     def get_default_data(self):

#         m0, c0,= self.m0,self.c0

#         default_data_dict = {
#             self.m: [m0*no for no in range (1,8)],

#             self.c: [c0*no for no in range (0.3,1)],

#         }

#         return default_data_dict

    @property
    def _report_components(self):
        
        comp_list=[
       *REPORT_COMPONENTS_LIST
        ]
        
        return comp_list
    
class MissileTrajectory(MissileTrajectoryAirless):

    m=Symbol('m',positive=True)
    g=Symbol('g',positive=True)
    c=Symbol('c',positive=True)
    x=dynamicsymbols('x',positive=True)
    y=dynamicsymbols('y',positivie=True)
    c0=Symbol('c0',positive=True)
    ivar=Symbol('t')

    
    def __init__(self,
                 m=None,
                 g=None,
                 c=None,
                 x=None,
                 y=None,
                 ivar=None,
                 **kwargs):
        
        if m is not None: self.m = m
        if g is not None: self.g = g
        if c is not None: self.c = c
        if x is not None: self.x = x
        if y is not None: self.y = y
        if ivar is not None: self.ivar= ivar

        self.qs = [self.x,self.y]
        
        self._init_from_components(**kwargs)


        
    @property
    def components(self):

        v_x=(self.x).diff(self.ivar)
        v_y=(self.y).diff(self.ivar)
        components = {}

        self._missile_airless = MissileTrajectoryAirless(self.m, x=self.x, y=self.y, qs=self.qs)

        self._drag_x = Damper(self.c,self.x,qs=self.qs)(label = 'horizontal drag')
         
        self._drag_y = Damper(self.c,self.y,qs=self.qs)(label = 'drag')
        
        

        
        components['_missile_airless'] = self._missile_airless
        components['_drag_x'] = self._drag_x
        components['_drag_y'] = self._drag_y
        
        return components

#     def get_default_data(self):

#         m0, c0,= self.m0,self.c0

#         default_data_dict = {
#             self.m: [m0*no for no in range (1,8)],

#             self.c: [c0*no for no in range (1,8)],

#         }
        
#         return default_data_dict
        
#     def get_numerical_data(self):

#         default_data_dict = {
#             self.m: [140],
#             self.g: [10],
#             self.c: [0.95],
#         }
#         return default_data_dict

    
class MissileTrajectoryAerodynamic(MissileTrajectory):
    
    m=Symbol('m',positive=True)
    g=Symbol('g',positive=True)
    c=Symbol('c',positive=True)
    x=dynamicsymbols('x',positive=True)
    y=dynamicsymbols('y',positivie=True)
    c=Symbol('c',positive=True)
    ro=Symbol('\\rho',positive=True)
    area=Symbol('A',positive=True)
    c_x=Symbol('c_x',positive=True)
    c0=Symbol('c0',positive=True)
    
    
    def __init__(self,
                 m=None,
                 g=None,
                 c=None,
                 x=None,
                 y=None,
                 ca=None,
                 ro=None,
                 area=None,
                 c_x=None,
                 ivar=None,
                 **kwargs):
        
        if m is not None: self.m = m
        if g is not None: self.g = g
        if c is not None: self.c = c
        if x is not None: self.x = x
        if y is not None: self.y = y
        if ca is not None: self.ca = ca
        if ro is not None: self.ro = ro
        if area is not None: self.area = area
        if c_x is not None: self.c_x = c_x
        if ivar is not None: self.ivar = ivar

        self.qs = [self.x,self.y]
        
        self._init_from_components(**kwargs)
       
    @property
    def components(self):
        area = self.area
        v_x=(self.x).diff(self.ivar)
        v_y=(self.y).diff(self.ivar)
        ca = 1/2*self.ro*area*self.c_x*((((self.x).diff(self.ivar))**2+((self.y).diff(self.ivar))**2))
        components = {}

        self.missile = MissileTrajectory(m = self.m, g = self.g, c = ca, x=self.x, y=self.y, qs=self.qs)
 
        components['missile'] = self.missile

        return components

#     def get_default_data(self):

#         m0, c0,= self.m0,self.c0

#         default_data_dict = {
#             self.m: [m0*no for no in range (1,8)],

#             self.c: [c0*no for no in range (1,8)],

#         }
        
#         return default_data_dict
        
#     def get_numerical_data(self):

#         default_data_dict = {
#             self.m: [140],
#             self.g: [10],
#             self.c: [0.95],
#             self.area:[1]
#             self.ro:[1]
#             selfc_x:[1]
#             }
        

#         return default_data_dict


from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin
from  ..continuous import ContinuousSystem, PlaneStressProblem

import base64
import random
import IPython as IP
import numpy as np
import inspect

from .principles import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin, REPORT_COMPONENTS_LIST


class BlowerToothedBelt(ComposedSystem):

    scheme_name = 'blower_toothed_belt.png'
    real_name = 'blown_440_big_block.jpg'
    detail_scheme_name = 'blower_roller_bolt.png'
    detail_real_name = 'tensioner_pulley.jpg'

    m=Symbol('m', positive=True)
    k_belt=Symbol('k_b', positive=True)
    k_tensioner=Symbol('k_t', positive=True)
    ivar=Symbol('t')
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    z0 = Symbol('z_0', positive=True)
    z=dynamicsymbols('z')


    def __init__(self,
                 m=None,
                 k_belt=None,
                 k_tensioner=None,
                 ivar=Symbol('t'),
                 Omega=None,
                 F=None,
                 z0=None,
                 z=None,
                 **kwargs):

        if m is not None: self.m = m
        if k_belt is not None: self.k_belt = k_belt
        if k_tensioner is not None: self.k_tensioner = k_tensioner
        if F is not None: self.F = F
        if Omega is not None: self.Omega = Omega
        if z is not None: self.z = z
        if ivar is not None: self.ivar = ivar
        if z0 is not None: self.z0 = z0

        self.ivar = ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        self._mass = MaterialPoint(self.m, self.z, qs=[self.z])(label = 'Material point')
        self._upper_belt = Spring(self.k_belt, self.z, pos2=0, qs=[self.z])(label = 'Upper belt stiffness')
        self._lower_belt = Spring(self.k_belt, self.z, pos2=0, qs=[self.z])(label = 'Lower belt stiffness')
        self._tensioner = Spring(self.k_tensioner, self.z, pos2=self.z0, qs=[self.z])(label = 'Tensioner stiffness')
        self._force = Force(self.F * sin(self.Omega * self.ivar), pos1=self.z, qs=[self.z])(label = 'Force')



        components['_mass'] = self._mass
        components['_upper_belt'] = self._upper_belt
        components['_lower_belt'] = self._lower_belt
        components['_tensioner'] = self._tensioner
        components['_force'] = self._force
        return components

    def get_default_data(self):


        k0, F0 = symbols('k0 F0', positive=True)

        default_data_dict = {
            self.F: [F0, 2 * F0, 3 * F0, 4 * F0, 5 * F0, 6 * F0],
            self.z0: [F0 / k0 * S.One / 4 * no for no in range(1, 2)],
        }

        return default_data_dict
    
    def get_numerical_data(self):

        default_data_dict = {
            self.m: [2 * no for no in range(1, 8)],
            self.k_belt: [10e3 * no for no in range(1, 8)],
            self.k_tensioner: [10e5 * no for no in range(4, 8)],
            self.F: [500 * no for no in range(1, 8)],
            self.Omega: [0.02*no for no in range(1,4)],
           # self.Q: [200* no for no in range(1,8)],
            self.z0: [0.2 * no for no in range(1, 2)],
        }

        return default_data_dict

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of the system on the spring',
            self.k_belt: r'Belt stiffness factor',
            self.k_tensioner: r'Tensioner spring stiffnessfactor',
            self.Omega: r'Angle of exciting function',
            self.F: r'Exciting force',
           # self.Q: r'Load of the system',
            self.z: r'Displacement of the system'
        }

        return self.sym_desc_dict

    @property
    def _report_components(self):
        
        comp_list=[
        *REPORT_COMPONENTS_LIST,
        mech_comp.TensionerForce,
        mech_comp.MaxStaticForce,
        mech_comp.MaxDynamicForce,
        mech_comp.StaticPinDiameter,
        mech_comp.DynamicPinDiameter
        ]

        return comp_list
    
    def tensioner_belt_force(self):
        return self.k_tensioner * self.steady_solution()[0]

    def left_belt_force(self):
        return self.k_belt * self.steady_solution()[0]

    def right_belt_force(self):
        return self.k_belt * self.steady_solution()[0]

    def max_static_force(self):
        data=self._given_data
        ans=self.dynamic_force()
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
#        abs(self.static_load().doit()[0])
        return abs(free_coeff)

    
    def dynamic_force(self):

        amps = self._fodes_system.steady_solution.as_dict()
        force_in_tensioner = self.components['_tensioner'].force().doit().expand()#.doit()
        data=self._given_data

        ##### new force implementation
        ans = force_in_tensioner.subs(amps).expand().doit()

        return ans.subs(data)
    
    
    def max_dynamic_force(self):

        ans=self.dynamic_force()
        data=self._given_data
        sin_coeff=ans.coeff(sin(self.Omega*self.ivar)).subs(data)
        #display(sincoeff)
        cos_coeff=ans.coeff(cos(self.Omega*self.ivar)).subs(data)
        #display(coscoeff)
        free_coeff=ans.subs({cos(self.Omega*self.ivar):0, sin(self.Omega*self.ivar):0}).subs(data)
        #display(freecoeff)
        return sqrt(sin_coeff**2 + cos_coeff**2) + abs(free_coeff)

    def static_force_pin_diameter(self):
        kt = Symbol('k_shear', positive=True)
        Re = Symbol('R_e', positive=True)
        return ((4 * self.max_static_force()) / (pi * kt * Re))**(1 / 2)

    def dynamic_force_pin_diameter(self):
        kt = Symbol('k_shear', positive=True)
        Re = Symbol('R_e', positive=True)
        data=self._given_data
        load = abs(self.max_dynamic_force())
        return (((4 * load) / (pi * kt * Re))**(1 / 2)).subs(data)




class DampedBlowerToothedBelt(BlowerToothedBelt):

    scheme_name = 'damped_blower_toothed_belt.png'
    real_name = 'blown_440_big_block.jpg'
    detail_scheme_name = 'blower_roller_bolt.png'
    detail_real_name = 'tensioner_pulley.jpg'

    m=Symbol('m', positive=True)
    k_belt=Symbol('k_b', positive=True)
    k_tensioner=Symbol('k_t', positive=True)
    ivar=Symbol('t')
    Omega=Symbol('Omega', positive=True)
    F=Symbol('F', positive=True)
    z=dynamicsymbols('z')
    c_belt = Symbol('c_b', positive=True)
    c_tensioner = Symbol('c_t', positive=True)
    lam = Symbol('lambda', positive=True)
    z0 = Symbol('z0', positive=True)

    def __init__(
            self,
            m=None,
            k_belt=None,
            k_tensioner=None,
            ivar=Symbol('t'),
            Omega=None,
            F=None,
            z0=None,
            z=None,
            c_belt=None,
            c_tensioner=None,
            lam=None,
            qs=None,
            **kwargs):

        if m is not None: self.m = m
        if k_belt is not None: self.k_belt = k_belt
        if k_tensioner is not None: self.k_tensioner = k_tensioner
        if F is not None: self.F = F
        if Omega is not None: self.Omega = Omega
        if z is not None: self.z = z
        if ivar is not None: self.ivar = ivar
        if z0 is not None: self.z0 = z0
        if qs is not None: self.qs = qs
        else: self.qs = [self.z]
        if lam is not None: self.lam = lam
        if c_belt is not None: self.c_belt = c_belt
        if c_tensioner is not None: self.c_tensioner = c_tensioner

        self.ivar = ivar

        self._init_from_components(**kwargs)

    @property
    def components(self):

        components = {}

        #self._blower_toothed_belt = BlowerToothedBelt(m=self.m, k_belt=self.k_belt, k_tensioner=self.k_tensioner, ivar=self.ivar, Omega=self.Omega, F=self.F, z0=self.z0, z=self.z)(label = 'Blower Toothed Belt')
        #components['_blower_toothed_belt'] = self._blower_toothed_belt
        
        self._mass = MaterialPoint(self.m, self.z, qs=[self.z])(label = 'Material point')
        self._upper_belt = Spring(self.k_belt, self.z, pos2=0, qs=[self.z])(label = 'Upper belt stiffness')
        self._lower_belt = Spring(self.k_belt, self.z, pos2=0, qs=[self.z])(label = 'Lower belt stiffness')
        self._tensioner = Spring(self.k_tensioner, self.z, pos2=self.z0, qs=[self.z])(label = 'Tensioner stiffness')
        self._force = Force(self.F * sin(self.Omega * self.ivar), pos1=self.z, qs=[self.z])(label = 'Force')



        components['_mass'] = self._mass
        components['_upper_belt'] = self._upper_belt
        components['_lower_belt'] = self._lower_belt
        components['_tensioner'] = self._tensioner
        components['_force'] = self._force
        
        
        self._upper_belt_damping = Damper(self.c_belt, pos1=self.z, pos2=0, qs=[self.z])(label = 'Upper belt damping')
        self._lower_belt_damping = Damper(self.c_belt, pos1=self.z, pos2=0, qs=[self.z])(label = 'Lower belt damping')
        self._tensioner_damping = Damper(self.c_tensioner, pos1=self.z, pos2=self.z0, qs=[self.z])(label = 'Tensioner damping')


        components['_upper_belt_damping'] = self._upper_belt_damping
        components['_lower_belt_damping'] = self._lower_belt_damping
        components['_tensioner_damping'] = self._tensioner_damping

        return components

    def get_default_data(self):

        default_data_dict = {

            self.c_belt: [self.lam * (self.k_belt)],
            self.c_tensioner: [self.lam * (self.k_tensioner)],
        }

        return default_data_dict
    
    def reference_data_dict(self):

        lam, m0, k0, l0, F0 = symbols('lambda m_0 k_0 l_0 F_0', positive=True)

        default_data_dict = {
            self.c_belt: [self.lam*(self.k_belt)],
            self.c_tensioner: [self.lam*(self.k_tensioner)],
        }

        return default_data_dict

    def get_numerical_data(self):

        default_data_dict = {
            self.c_belt: [(self.k_belt)*self.lam],
            self.c_tensioner: [(self.k_tensioner)*self.lam],

            self.m: [2 * no for no in range(1, 8)],
            self.k_belt: [1000 * no for no in range(1, 8)],
            self.k_tensioner: [10000 * no for no in range(4, 8)],
            self.F: [500 * no for no in range(1, 8)],
            self.Omega: [2*3.14*no/4 for no in range(1,4)],
            self.lam: [no/50 for no in range(7,13)],
            self.z0: [0.2 * no for no in range(1, 2)],
        }
        return default_data_dict

    def symbols_description(self):
        self.sym_desc_dict = {
            self.m: r'Mass of the system on the spring',
            self.k_belt: r'Belt stiffness factor',
            self.k_tensioner: r'Tensioner spring stiffness factor',
            self.c_belt: r'Belt damping factor',
            self.c_tensioner: r'Tensioner damping factor',
            self.Omega: r'Angle of exciting function',
            self.F: r'Exciting force',
            self.z: r'Displacement of the system'
        }

        return self.sym_desc_dict

    @property
    def _report_components(self):

        comp_list=[
        *REPORT_COMPONENTS_LIST,
        mech_comp.TensionerForce,
        mech_comp.TensionerDamperForce,
        mech_comp.MaxStaticForce,
        mech_comp.MaxDynamicForce,
        mech_comp.StaticPinDiameter,
        mech_comp.DynamicPinDiameter,
        mech_comp.DampedVibrationFrequency
        ]

        return comp_list


    def tensioner_damper_force(self):
        return self.c_tensioner * self.steady_solution()[0].diff(self.ivar)

    def left_belt_damper_force(self):
        return self.c_belt * self.steady_solution()[0].diff(self.ivar)

    def right_belt_damper_force(self):
        return self.c_belt * self.steady_solution()[0].diff(self.ivar)
